#
# Licence GPL v3.
#
# Copyright Chris Johns (chrisj@rtems.org)
#
import optparse
import os
import pq
import pqconfig
import sys
import time

def publish_header(output, title):
    print >> output, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">'
    print >> output, '<html>'
    print >> output, ' <head>'
    print >> output, '  <title>' + title + '</title>'
    print >> output, '  <meta http-equiv="Content-Language" content="English" />'
    print >> output, '  <meta http-equiv="Content-Type" content="text/html; charset=us-ascii" />'
    ss = pqconfig.stylesheet()
    if ss:
        print >> output, '  <link rel="stylesheet" type="text/css" href="' + ss + '" media="screen" />'
    print >> output, ' </head>'
    print >> output, ' <body>'

def publish_footer(output):
    print >> output, ' </body>'
    print >> output, '</html>'

def publish_heading(output, heading):
    print >> output, '  <div class="heading">'
    print >> output, '   ' + heading
    print >> output, '   <div class="datetime">' + time.ctime() + '</div>'
    print >> output, '  </div>'

def publish_table_open(output, _class, cellspacing = 0):
    print >> output, '  <table class="' + _class + '" cellspacing="' + \
        str(cellspacing) + '" cellpadding="2">'

def publish_table_close(output):
    print >> output, '  </table>'

def publish_error_header(output):
    print >> output, '   <tr class="error-table-top"><th>Target</th>',
    print >> output, '<th>Dups</th><th>E</th><th>File</th><th>Error/Warning</th></tr>'

def publish_error(output, target, error, table_on):
    if table_on:
        print >> output, '    <tr class="error-table-on">',
    else:
        print >> output, '    <tr class="error-table-off">',
    print >> output, '<td>', pq.target_name(target, 3), '</td>',
    print >> output, '<td class="error-table-dups">', error['dup'], '</td>',
    if error['error'].type == 'error':
        print >> output, '<td class="error-table-error">&nbsp;</td>',
    else:
        print >> output, '<td class="error-table-warning">&nbsp;</td>',
    source_file = pqconfig.publish_source_file(error['error'].file, error['error'].line)
    if source_file:
        print >> output, '<td>', source_file, '</td>',
    else:
        print >> output, '<td>', '%s:%d' % (error['error'].file, error['error'].line), '</td>',
    print >> output, '<td>', error['error'].info, '</td>',
    print >> output, '</tr>'

def publish_navbar(output, first, prev, next, last):
    print >> output, '  <div class="navbar">'
    print >> output, '   <a href=".">Summary</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
    if first:
        print >> output, '   <a href="./' + first + '">First</a>'
    else:
        print >> output, '   First'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if prev:
        print >> output, '   <a href="./' + prev + '">Prev</a>'
    else:
        print >> output, '   Prev'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if next:
        print >> output, '   <a href="./' + next + '">Next</a>'
    else:
        print >> output, '   Next'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if last:
        print >> output, '   <a href="./' + last + '">Last</a>'
    else:
        print >> output, '   Last'
    print >> output, '  </div>'
    
def publish_target_stats(output, workspace, target, history):
    publish_table_open(output, 'stats-table', 1)
    print >> output, '   <tr class="stats-table-target"><td colspan="7">',
    print >> output, pq.target_name(target, 3), '</td></tr>'
    print >> output, '   <tr class="stats-table-top"><th>Date</th>',
    print >> output, '<th>Errors</th><th>Warnings</th><th>Notes</th>',
    print >> output, '<th>Duplicates</th><th>Status</th><th>Results</th></tr>'
    for h in history.history:
        if h[0] == target:
            print >> output, '   <tr class="stats-table-row">',
            print >> output, '<td align="center">', h[1], '</td>',
            print >> output, '<td>', h[2], '</td>',
            print >> output, '<td>', h[3], '</td>',
            print >> output, '<td>', h[4], '</td>',
            print >> output, '<td>', h[5], '</td>',
            if h[6] == 'PASSED':
                _class = 'stats-target-good'
            else:
                _class = 'stats-target-bad'
            print >> output, '<td class="' + _class + '">', h[6], '</td>',
            print >> output, '<td class="stats-target-results">'
            if h[7]:
                print >> output, '<a href="' + h[7] + '">Results</a>',
            print >> output, '</td></tr>'
    publish_table_close(output)

def queue_publish(base, data, workspace, source, targets, output):
    history = pq.history(data, output)
    pqconfig.publish_config_setup(base, output)
    index = open(os.path.join(output, 'index.html'), 'w') 
    publish_header(index, pqconfig.title())
    publish_heading(index, pqconfig.title())
    for target in targets:
        try:
            errors = pq.errors(base, data, source, pqconfig.source_find)
            stderr = open(os.path.join(workspace, target, 'stderr.txt'))
            errors.load_stderr(target, stderr)
            stderr.close()
        except IOError, e:
            print >> sys.stderr, "cannot open the target stderr:", target
            continue
        t = time.time()
        ofile = None
        count = 0
        new_page = True
        page = 0
        pages = (len(errors.stderr) / pqconfig.errors_per_page())
        filename_base = pqconfig.publish_filename(target, t)
        first_filename = filename_base + '-1.html'
        last_filename = filename_base + '-%d.html' % (pages)
        prev_filename = None
        next_filename = None
        filename = None
        for error in errors.walk_stderr():
            if errors.check_all(target, error['error'].error):
                continue
            if new_page:
                count = 0
                page += 1
                new_page = False
                if ofile:
                    publish_table_close(ofile)
                    publish_navbar(ofile, first_filename, 
                                   prev_filename, next_filename,
                                   last_filename)
                    publish_footer(ofile)
                    ofile.close()
                prev_filename = filename
                filename = filename_base + '-%d.html' % (page)
                if not prev_filename:
                    first_page = filename
                if page < pages:
                    next_filename = filename_base + '-%d.html' % (page + 1)
                else:
                    next_filename = None
                ofile = open(os.path.join(output, filename), 'w')
                publish_header(ofile, pqconfig.target_title(target, page))
                publish_heading(ofile, pqconfig.target_title(target, page))
                publish_table_open(ofile, 'error-table')
                publish_error_header(ofile)
                table_on = False
            publish_error(ofile, target, error, table_on)
            table_on = not table_on
            count += 1
            if count >= pqconfig.errors_per_page():
                new_page = True
        if ofile:
            publish_table_close(ofile)
            publish_navbar(ofile, first_filename, 
                           prev_filename, next_filename,
                           last_filename)
            publish_footer(ofile)
            ofile.close()
        history.add(workspace, target, errors, first_page, t)
        history.remove_oldest(target)
        publish_target_stats(index, workspace, target, history)
        del errors
    publish_footer(index)
    index.close()
    history.unload(data)

#
# Command line options.
#
cl_options = [
    optparse.make_option('-o', action = 'store', type = 'string', 
                         dest = 'output', default = None,
                         help = 'Directory to write the HTML output to'),
    # Standard options
    optparse.make_option('-d', action = 'store', type = 'string', 
                         dest = 'data', default = os.environ['HOME'],
                         help = 'Specify the data directory'),
    optparse.make_option('-w', action = 'store', type = 'string', 
                         dest = 'workspace', default = None,
                         help = 'Specify the workspace directory'),
    optparse.make_option('-s', action = 'store', type = 'string', 
                         dest = 'source', default = None,
                         help = 'Specify the source directory'),
    optparse.make_option('-t', action = 'store', type = 'string', 
                         dest = 'targets', default = None,
                         help = 'List of comma seperated targets'),
    optparse.make_option('-c', action = 'store', type = 'string',
                         dest = 'config',
                         default = os.environ['HOME'] + '/.pq-config',
                         help = 'Configration file name to read')
]

def user_main(base, argv):
    (options, config, data, workspace, source, targets) = \
        pq.parse_std_args(argv, cl_options)
    if not options.output:
        print >> sys.stderr, 'no output directory specified'
        sys.exit(1)
    queue_publish(base, data, workspace, source, targets, options.output)
